==> infinite-viewer-0.21.0.tgz: package/src/ScrollBar.ts <==
import Gesto from "gesto";
import {
    SCROLL_BAR_CLASS_NAME, SCROLL_THUMB_CLASS_NAME,
    HORIZONTAL_SCROLL_BAR_CLASS_NAME, VERTICAL_SCROLL_BAR_CLASS_NAME
} from "./consts";
import { addClass, removeEvent, addEvent } from "@daybrush/utils";
import EventEmitter from "@scena/event-emitter";

export default class ScrollBar extends EventEmitter {
    public isAppend: boolean = false;
    public thumbElement!: HTMLElement;
    public barElement!: HTMLElement;
    private gesto!: Gesto;
    private pos: number = 0;
    private size: number = 0;
    private scrollSize: number = 0;
    private isHorizontal = false;

    constructor(
        private type: "horizontal" | "vertical",
        container?: HTMLElement,
    ) {
        super();
        const isHorizontal = type === "horizontal";
        let thumbElement: HTMLElement;
        let barElement: HTMLElement = container;

        if (!container) {
            barElement = document.createElement("div");
            thumbElement = document.createElement("div");

            barElement.insertBefore(thumbElement, null);
            this.isAppend = true;
        } else {
            thumbElement = container.querySelector(`.${SCROLL_THUMB_CLASS_NAME}`);
        }
        addClass(
            barElement,
            isHorizontal ? HORIZONTAL_SCROLL_BAR_CLASS_NAME
                : VERTICAL_SCROLL_BAR_CLASS_NAME,
        );
        addClass(barElement, SCROLL_BAR_CLASS_NAME);
        addClass(thumbElement, SCROLL_THUMB_CLASS_NAME);

        this.thumbElement = thumbElement;
        this.barElement = barElement;
        this.isHorizontal = isHorizontal;
        this.gesto = new Gesto(barElement, {
            container: window,
        }).on("dragStart", e => {
            const target = e.inputEvent.target;
            const datas = e.datas;
            const isThumb = this.thumbElement === target;

            if (!isThumb) {
                setTimeout(() => {
                    requestAnimationFrame(() => {
                        const thumbRect = this.thumbElement.getBoundingClientRect();
                        const pos1 = thumbRect[isHorizontal ? "left" : "top"];
                        const thumbSize = thumbRect[isHorizontal ? "width" : "height"];
                        const pos2 = pos1 + thumbSize;
                        const clientPos = e[isHorizontal ? "clientX" : "clientY"];

                        if (pos1 <= clientPos && clientPos <= pos2) {
                            return;
                        }
                        const size = this.size;
                        const delta = size * size / this.scrollSize;

                        this.scrollBy(pos1 < clientPos ? delta : -delta);
                    });
                }, 100);

            }
            datas.isThumb = isThumb;
            e.inputEvent.stopPropagation();
            e.inputEvent.preventDefault();
        }).on("drag", e => {
            if (!e.datas.isThumb) {
                return;
            }
            this.scrollBy(this.isHorizontal ? e.deltaX : e.deltaY);
        });
        addEvent(this.barElement, "wheel", this.onWheel, {
            passive: false,
        });
    }
    public scrollBy(delta: number) {
        const ratio = delta / this.size;

        this.trigger("scroll", {
            delta: this.scrollSize * ratio,
        });
    }
    public render(isDisplay: boolean, pos: number, size: number, scrollSize: number) {
        this.pos = pos;
        this.size = size;
        this.scrollSize = scrollSize;

        const display = isDisplay && (scrollSize > size) ? "block" : "none";
        const [dirName1, sizeName] = this.isHorizontal ? ["X", "width"] : ["Y", "height"];

        this.barElement.style.cssText = `display: ${display};`;
        this.thumbElement.style.cssText
            += `${sizeName}: ${size * size / scrollSize}px;`
            + `transform: translate${dirName1}(${pos / scrollSize * size}px)`;
    }
    public destroy() {
        removeEvent(this.barElement, "wheel", this.onWheel);
        this.gesto.off();
        this.off();
    }
    private onWheel = (e: WheelEvent) => {
        const delta = this.isHorizontal ? e.deltaX : e.deltaY;

        if (delta) {
            e.preventDefault();
        }
        this.trigger("scroll", {
            delta,
        });
    }
}

==> infinite-viewer-0.23.0.tgz: package/src/ScrollBar.ts <==
import Gesto from "gesto";
import {
    SCROLL_BAR_CLASS_NAME, SCROLL_THUMB_CLASS_NAME,
    HORIZONTAL_SCROLL_BAR_CLASS_NAME, VERTICAL_SCROLL_BAR_CLASS_NAME
} from "./consts";
import { addClass, removeEvent, addEvent, throttle } from "@daybrush/utils";
import EventEmitter from "@scena/event-emitter";
import { abs } from "./utils";

export default class ScrollBar extends EventEmitter {
    public isAppend: boolean = false;
    public thumbElement!: HTMLElement;
    public barElement!: HTMLElement;
    private gesto!: Gesto;
    private size: number = 0;
    private scrollSize: number = 0;
    private isHorizontal = false;

    constructor(
        private type: "horizontal" | "vertical",
        container?: HTMLElement,
    ) {
        super();
        const isHorizontal = type === "horizontal";
        let thumbElement: HTMLElement;
        let barElement: HTMLElement = container;

        if (!container) {
            barElement = document.createElement("div");
            thumbElement = document.createElement("div");

            barElement.insertBefore(thumbElement, null);
            this.isAppend = true;
        } else {
            thumbElement = container.querySelector(`.${SCROLL_THUMB_CLASS_NAME}`);
        }
        addClass(
            barElement,
            isHorizontal ? HORIZONTAL_SCROLL_BAR_CLASS_NAME
                : VERTICAL_SCROLL_BAR_CLASS_NAME,
        );
        addClass(barElement, SCROLL_BAR_CLASS_NAME);
        addClass(thumbElement, SCROLL_THUMB_CLASS_NAME);

        this.thumbElement = thumbElement;
        this.barElement = barElement;
        this.isHorizontal = isHorizontal;
        this.gesto = new Gesto(barElement, {
            container: window,
        }).on("dragStart", e => {
            const target = e.inputEvent.target;
            const datas = e.datas;
            const isThumb = this.thumbElement === target;

            if (!isThumb) {
                setTimeout(() => {
                    requestAnimationFrame(() => {
                        const thumbRect = this.thumbElement.getBoundingClientRect();
                        const pos1 = thumbRect[isHorizontal ? "left" : "top"];
                        const thumbSize = thumbRect[isHorizontal ? "width" : "height"];
                        const pos2 = pos1 + thumbSize;
                        const clientPos = e[isHorizontal ? "clientX" : "clientY"];

                        const endPos = clientPos - pos2;
                        const startPos = clientPos - pos1;

                        if (0 <= startPos && endPos <= 0) {
                            return;
                        }
                        // thumbSize === containerSize
                        // scrollWidth =
                        this.scrollSize
                        const clientScrollWidth = thumbSize / this.size * this.scrollSize;
                        const pos = (0 < endPos ? endPos : startPos) / clientScrollWidth;
                        const delta = pos * this.size;

                        console.log(this.scrollSize, pos, delta);

                        this.scrollBy(delta);
                    });
                }, 100);

            }
            datas.isThumb = isThumb;
            e.inputEvent.stopPropagation();
            e.inputEvent.preventDefault();
        }).on("drag", e => {
            if (!e.datas.isThumb) {
                return;
            }
            this.scrollBy(this.isHorizontal ? e.deltaX : e.deltaY);
        });
        addEvent(this.barElement, "wheel", this.onWheel, {
            passive: false,
        });
    }
    public scrollBy(delta: number) {
        const ratio = delta / this.size;

        this.trigger("scroll", {
            delta: this.scrollSize * ratio,
        });
    }
    public render(
        isDisplay: boolean,
        containerSize: number,
        scrollRange: number[],
    ) {
        const [startMargin, endMargin] = scrollRange;
        const scrollSizeOffset = throttle(abs(startMargin) + endMargin, 0.001);
        const scrollSize = containerSize + scrollSizeOffset;
        const display = isDisplay && scrollSizeOffset ? "block" : "none";
        const [dirName1, sizeName] = this.isHorizontal ? ["X", "width"] : ["Y", "height"];
        const thumbSize = containerSize / scrollSize;
        const thumbPos = endMargin / scrollSize / thumbSize;

        this.size = containerSize;
        this.scrollSize = scrollSize;
        this.barElement.style.cssText = `display: ${display};`;
        this.thumbElement.style.cssText
            += `${sizeName}: ${thumbSize * 100}%;`
            + `transform: translate${dirName1}(${thumbPos * 100}%)`;
    }
    public destroy() {
        removeEvent(this.barElement, "wheel", this.onWheel);
        this.gesto.off();
        this.off();
    }
    private onWheel = (e: WheelEvent) => {
        const delta = this.isHorizontal ? e.deltaX : e.deltaY;

        if (delta) {
            e.preventDefault();
        }
        this.trigger("scroll", {
            delta,
        });
    }
}
